# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-07-30

"""
Provides download functions for FTP/SFTP, HTTP and local data sources.
"""
import calendar
from datetime import datetime
from ftplib import FTP, socket
import os
import shutil
import paramiko
import requests
import pandas as pd
from poets.timedate.dekad import dekad2day, runningdekad2date
from requests.exceptions import ConnectionError, Timeout


def download_ftp(download_path, host, directory, filedate, port=21,
                 username='', password='', dirstruct=None, ffilter='',
                 begin=None, end=None):
    """Downloads data via FTP.

    Parameters
    ----------
    download_path : str, optional
        Path where to save the downloaded files.
    host : str
        Link to host.
    directory : str
        Path to data on host.
    filedate : dict
        Dict which points to the date fields in the filename
    port : int, optional
        Port to host, defaults to 21.
    username : str, optional
        Username for source, defaults to emtpy str.
    password : str, optional
        Passwor for source, defaults to emtpy str.
    dirstruct : list of str, optional
        Folder structure on host, each list element represents a subdirectory.
    ffilter : str, optional
        Used for filtering files on a server, defaults to emtpy str.
    begin : datetime, optional
        Set either to first date of remote repository or date of last file in
        local repository.
    end : datetime, optional
        Date until which data should be downloaded.

    Returns
    -------
    bool
        True if data is available, False if not.
    """

    if begin is None:
        begin = datetime(1900, 1, 1)

    if end is None:
        end = datetime.now()

    if host[-1] == '/':
        host = host[:-1]

    try:
        ftp = FTP(host)
    except:
        print '[ERROR] Cannot connect to source. Please contact data provider.'
        return False

    try:
        ftp.login(username, password)
    except:
        print '[ERROR] Cannot login at source - wrong login credentials.'
        return False

    try:
        ftp.cwd(directory)
    except:
        print '[ERROR] Directory not found on host!'
        return False

    subdirs = []

    ftp.retrlines("NLST", subdirs.append)  # NLIST retrieves filename only

    if not os.path.exists(download_path):
        os.makedirs(download_path)

    os.chdir(download_path)
    print '[INFO] downloading data from ' + str(begin) + ' to ' + str(end)

    files = []

    if dirstruct is not None and len(dirstruct) > 0 and dirstruct[0] == 'YYYY':
        for year in subdirs:
            if begin.year > int(year):
                continue
            if end.year < int(year):
                continue
            year_filelist = []
            ftp.cwd(directory + year)
            ftp.retrlines("NLST", year_filelist.append)
            if len(dirstruct) == 2 and dirstruct[1] == ['MM', 'M']:
                mon_filelist = []
                for month in year_filelist:
                    if begin.year == int(year):
                        if begin.month > int(month):
                            continue
                    if end.year == int(year):
                        if end.month < int(month):
                            continue
                    ftp.cwd(directory + year + '/' + month)
                    ftp.retrlines("NLST", mon_filelist.append)
                files += mon_filelist
            else:
                files += year_filelist
    else:
        files = filesInDir_ftp(directory, ftp, filedate, begin, end, files)
        if files is False:
            print '[ERROR] Connection timed out, skipping source.'
            return False

    ftp.cwd(directory)
    if len(files) > 0:
        for fname in files:
            fname2 = fname.split('/')[-1]
            if not os.path.exists(os.path.join(download_path, fname2)):
                if ffilter is None or ffilter in fname2:
                    loc = '/'.join(fname.split('/')[:-1])
                    try:
                        ftp.cwd(loc)
                        ftp.retrbinary("RETR " + fname2,
                                       open(fname2, "wb").write)
                    except:
                        print '[WARNING] Connection timed out, retrying...'
                        try:
                            ftp.close()
                            ftp = FTP(host)
                            ftp.login(username, password)
                            ftp.cwd(directory)
                            ftp.cwd(loc)
                            ftp.retrbinary("RETR " + fname2,
                                           open(fname2, "wb").write)
                        except:
                            '[ERROR] Retry not successful, skipping download.'
                            return False
        ftp.close()
        return True
    else:
        ftp.close()
        return False

    return True


def download_sftp(download_path, host, directory, port, username, password,
                  filedate, dirstruct=None, ffilter='',
                  begin=None, end=None):
    """Download data via SFTP.

    Parameters
    ----------
    download_path : str, optional
        Path where to save the downloaded files.
    host : str
        Link to host.
    directory : str
        Path to data on host.
    port : int
        Port to host.
    username : str
        Username for source.
    password : str
        Password for source.
    filedate : dict
        Dict which points to the date fields in the filename.
    dirstruct : list of str, optional
        Folder structure on host, each list element represents a subdirectory.
    ffilter : str, optional
        Used for filtering files on a server, defaults to emtpy str.
    begin : datetime, optional
        Set either to first date of remote repository or date of last file in
        local repository.
    end : datetime, optional
        Date until which data should be downloaded.

    Returns
    -------
    bool
        True if data is available, false if not.
    """

    if begin is None:
        begin = datetime(1900, 1, 1)

    if end is None:
        end = datetime.now()

    if not os.path.exists(download_path):
        print('[INFO] output path does not exist... creating path')
        os.makedirs(download_path)

    print '[INFO] downloading data from ' + str(begin) + ' - ' + str(end)

    if not os.path.exists(download_path):
        os.makedirs(download_path)

    localpath = download_path

    # connect to ftp server
    if host[-1] == '/':
        host = host[:-1]

    try:
        transport = paramiko.Transport((host, port))
    except paramiko.ssh_exception.SSHException:
        print '[ERROR] Cannot connect to source. Please contact data provider.'
        return False

    try:
        transport.connect(username=username, password=password)
        sftp = paramiko.SFTPClient.from_transport(transport)
    except:
        print '[ERROR] Cannot login at source.'
        return False

    try:
        subdirs = sftp.listdir(directory)
    except:
        print '[ERROR] Directory not found on host!'
        return False

    files = []

    if dirstruct is not None and len(dirstruct) > 0 and dirstruct[0] == 'YYYY':
        for year in subdirs:
            if begin.year > int(year):
                continue
            if end.year < int(year):
                continue
            year_subdir = directory + str(year) + '/'
            year_filelist = sftp.listdir(year_subdir)
            year_filelist.sort()
            if len(dirstruct) == 1:
                for f in year_filelist:
                    files.append(year_subdir + f)
            elif len(dirstruct) > 1 and dirstruct[1] in ['MM', 'M']:
                for month in year_filelist:
                    if begin.year == int(year):
                        if begin.month > int(month):
                            continue
                    if end.year == int(year):
                        if end.month < int(month):
                            continue
                    mon_subdir = year_subdir + month + '/'
                    mon_filelist = sftp.listdir(mon_subdir)
                    mon_filelist.sort()
                    if len(dirstruct) == 2:
                        for f in mon_filelist:
                            files.append(mon_subdir + f)
                    elif dirstruct[2] in ['DD', 'D']:
                        for day in mon_filelist:
                            if begin.year == int(year):
                                if begin.month == int(month):
                                    if begin.day > int(day):
                                        continue
                            if end.year == int(year):
                                if begin.month == int(month):
                                    if end.day < int(day):
                                        continue
                            day_subdir = mon_subdir + day + '/'
                            day_filelist = sftp.listdir(day_subdir)
                            day_filelist.sort()
                            if len(dirstruct) == 2:
                                for f in day_filelist:
                                    files.append(day_subdir + f)
    else:
        files = filesInDir_sftp(directory, sftp, filedate, begin, end, files)

    if len(files) > 0:
        for f in files:
            filename = os.path.basename(f)
            fdate = get_file_date(filename, filedate)
            if fdate >= begin and fdate <= end:
                if ffilter is not None and ffilter not in f:
                    continue
                if os.path.isfile(os.path.join(localpath, filename)) is False:
                    try:
                        sftp.get(f, os.path.join(localpath, filename))
                    except:
                        print '[WARNING] Connection timed out, retrying...'
                        try:
                            sftp.close
                            transport = paramiko.Transport((host, port))
                            transport.connect(username=username,
                                              password=password)
                            sftp = (paramiko.SFTPClient
                                    .from_transport(transport))
                            sftp.get(f, os.path.join(localpath, filename))
                        except:
                            '[ERROR] Retry not successful, skipping download.'
                            return False
        sftp.close
        return True
    else:
        sftp.close
        return False


def download_http(download_path, host, directory, filename, filedate,
                  dirstruct, ffilter=None, begin=None,
                  end=datetime.now()):
    """Download data via HTTP

    Parameters
    ----------
    download_path : str, optional
        Path where to save the downloaded files.
    host : str
        Link to host.
    directory : str
        Path to data on host.
    filename : str
        Structure/convention of the file name.
    filedate : dict
        Dict which points to the date fields in the filename.
    dirstruct : list of str
        Folder structure on host, each list element represents a subdirectory.
    ffilter : str, optional
        Used for filtering files on a server, defaults to None.
    begin : datetime, optional
        Set either to first date of remote repository or date of last file in
        local repository.
    end : datetime, optional
        Date until which data should be downloaded.

    Returns
    -------
    bool
        true if data is available, false if not
    """

    if begin is None:
        begin = datetime(1900, 1, 1)

    if end is None:
        end = datetime.now()

    print('[INFO] downloading data from ' + str(begin) + ' - ' + str(end))

    # create daterange on monthly basis
    mon_from = datetime(begin.year, begin.month, 1)
    mon_to = datetime(end.year, end.month, 1)
    daterange = pd.date_range(start=mon_from, end=mon_to, freq='MS')

    if '{MM}' in filename:
        leading_month = True
    else:
        leading_month = False

    path = host + directory

    if not os.path.exists(download_path):
        os.makedirs(download_path)

    # loop through daterange
    for i, dat in enumerate(daterange):
        year = str(dat.year)
        month = str("%02d" % (dat.month,))

        if dirstruct is not None and dirstruct == ['YYYY']:
            path = host + directory + year + '/'
        elif dirstruct == ['YYYY', 'MM']:
            path = host + directory + year + '/' + month + '/'
        elif dirstruct == ['YYYY', 'M']:
            path = host + directory + year + '/' + dat.month + '/'
        else:
            path = host + directory  # + subdirectories!

        if leading_month is True:
            month = str("%02d" % (dat.month,))
            fname = filename.replace('{YYYY}', year).replace('{MM}', month)
        else:
            fname = filename.replace('{YYYY}', year).replace('{M}', month)

        files = []

        if '{P}' in filename:

            dekads = range(3)

            # get dekad of first and last interval based on input dates
            if begin.year == end.year and begin.month == end.month:
                if begin.day < 11:
                    if end.day > 10:
                        if end.day > 20:
                            dekads = range(3)
                        else:
                            dekads = [0, 1]
                    else:
                        dekads = [0]
                elif begin.day > 10 and begin.day < 21:
                    if end.day < 21:
                        dekads = [1]
                    elif end.day > 20:
                        dekads = [1, 2]
                else:
                    dekads = [2]
            else:
                if i == 0 and begin.day > 1:
                    if begin.day < 11:
                        dekads = [0, 1, 2]
                    elif begin.day >= 11 and begin.day < 21:
                        dekads = [1, 2]
                    elif begin.day == 21:
                        dekads = [2]
                elif i == (len(daterange) - 1) and end.day < 21:
                    if end.day < 11:
                        dekads = [0]
                    else:
                        dekads = [0, 1]

            # loop through dekads
            for j in dekads:
                filepath = path + fname.replace('{P}', str(j + 1))
                files.append(filepath)

        elif '{D}' in filename or '{DD}':

            if '{DD}' in filename:
                leading_day = True
            else:
                leading_day = False

            mr = calendar.monthrange(2014, 7)
            fday = mr[0]
            lday = mr[1] + 1

            if i == 0 and begin.day > 1:
                days = range(begin.day, lday)
            elif i == (len(daterange) - 1) and end.day < lday:
                days = range(fday, end.day + 1)
            else:
                days = range(fday, lday)

            # loop through dekads
            for j in days:
                if leading_day is True:
                    day = str("%02d" % (j))
                    filepath = path + fname.replace('{DD}', day)
                else:
                    filepath = path + fname.replace('{D}', str(j + 1))
                files.append(filepath)

        else:
            files.append(fname)

        for fp in files:
            newfile = os.path.join(download_path, fp.split('/')[-1])
            if os.path.exists(newfile):
                continue

            try:
                r = requests.get(fp, timeout=20.)
            except ConnectionError:
                print '[ERROR] Could not connect to resource, skipping...'
                continue
            except Timeout:
                print '[WARNING] File not available at resource, skipping...'
                continue

            if r.status_code == 200:
                # check if year folder is existing
                if ffilter is None or ffilter in os.path.split(fp)[-1]:
                    if not os.path.exists(download_path):
                        print('[INFO] output path does not exist...'
                              'creating path')
                        os.makedirs(download_path)

                    # download file
                    newfile = os.path.join(download_path,
                                           fp.split('/')[-1])
                    r = requests.get(fp, stream=True)
                    with open(newfile, 'wb') as f:
                        f.write(r.content)

    if len(files) > 0:
        return True
    else:
        return False


def download_local(download_path, directory, filedate, dirstruct=None,
                   ffilter='', begin=datetime(1900, 1, 1), end=datetime.now()):
    """Download data from local path

    Parameters
    ----------
    download_path : str
        Path where to save the downloaded files.
    directory : str
        Path to locally stored data.
    filedate : dict
        Dict which points to the date fields in the filename.
    dirstruct : list of str, optional
        Folder structure in directory, each list element represents
        a subdirectory.
    ffilter : str, optional
        Used for filtering files on a server, defaults to empty string.
    begin : datetime, optional
        Set either to first date of remote repository or date of last file in
        local repository, defaults to datetime(1900, 1, 1).
    end : datetime, optional
        Date until which data should be downloaded, defaults to datetime.now()

    Returns
    -------
    bool
        True if data is available, false if not.
    """

    if not os.path.exists(download_path):
        os.makedirs(download_path)

    files = []

    print '[INFO] downloading data from ' + str(begin) + ' to ' + str(end)

    # directory/folder (len(dirstruct) == 1)
    if dirstruct is not None and len(dirstruct) == 1:
        folders = os.listdir(directory)
        for folder in folders:
            filelist = os.listdir(os.path.join(directory, folder))
            for fname in sorted(filelist):
                date = get_file_date(fname, filedate)
                if date >= begin and date <= end:
                    files.append(fname)
                    if not os.path.exists(os.path.join(download_path,
                                                       fname)):
                        shutil.copy(os.path.join(directory, folder, fname),
                                    os.path.join(download_path, fname))

    # directory/folder/subfolder (len(dirstruct) == 2)
    elif dirstruct is not None and len(dirstruct) == 2:
        folders = os.listdir(directory)
        for folder in folders:
            subfolders = os.listdir(os.path.join(directory, folder))
            for subfolder in subfolders:
                filelist = os.listdir(os.path.join(directory, folder,
                                                   subfolder))
                for fname in sorted(filelist):
                    if date >= begin and date <= end:
                        files.append(fname)
                        if not os.path.exists(os.path.join(download_path,
                                                           fname)):
                            shutil.copy(os.path.join(directory, folder,
                                                     subfolder, fname),
                                        os.path.join(download_path, fname))

    # general case: unknown folder structure
    if dirstruct is None:
        for path, _, filelist in os.walk(directory):
            for fname in filelist:
                if ffilter is None or ffilter in fname:
                    date = get_file_date(fname, filedate)
                    if date >= begin and date <= end:
                        files.append(fname)
                        if not os.path.exists(os.path.join(download_path,
                                                           fname)):
                            shutil.copy(os.path.join(path, fname),
                                        os.path.join(download_path, fname))

    if len(files) > 0:
        return True
    else:
        return False


def get_file_date(fname, fdate):
    """Gets the date from a file name.

    Parameters
    ----------
    fname : str
        Filename.
    fdate : str
        Structure of the date in filename, dict which points to the date fields
        in the filename.

    Returns
    -------
    datetime
        Date and, if given, time from filename
    """

    fname = str(fname)

    if 'YYYY' in fdate.keys():
        year = int(fname[fdate['YYYY'][0]:fdate['YYYY'][1]])

    if 'MM' in fdate.keys():
        month = int(fname[fdate['MM'][0]:fdate['MM'][1]])

    if 'DD' in fdate.keys():
        day = int(fname[fdate['DD'][0]:fdate['DD'][1]])
    else:
        day = 1

    if 'P' in fdate.keys():
        dekad = int(fname[fdate['P'][0]:fdate['P'][1]])
        day = dekad2day(year, month, dekad)

    if 'RP' in fdate.keys():
        dat = runningdekad2date(year,
                                int(fname[fdate['RP'][0]:fdate['RP'][1]]))
        month = dat.month
        day = dat.day

    if 'hh' in fdate.keys():
        hour = int(fname[fdate['hh'][0]:fdate['hh'][1]])
    else:
        hour = 0

    if 'mm' in fdate.keys():
        minute = int(fname[fdate['mm'][0]:fdate['mm'][1]])
    else:
        minute = 0

    if 'ss' in fdate.keys():
        second = int(fname[fdate['ss'][0]:fdate['ss'][1]])
    else:
        second = 0

    return datetime(year, month, day, hour, minute, second)


def filesInDir_sftp(path, sftp, filedate, begin, end, filelist):
    """List all files in directory and subdirectories on an SFTP server.

    Parameters
    ----------
    path : str
        Path to data on host.
    sftp : paramiko Transport
        Connection to Server.
    filedate : dict
        Dict which points to the date fields in the filename.
    begin : datetime,
        Date from which on to download data.
    end : datetime
        Date until which to download data.
    filelist : list of str
        List of filepaths or empty list.

    Returns
    -------
    filelist : list
        List containing all files in directory and subdirectories
    """

    files = sorted(sftp.listdir(path))

    for fname in files:
        lstatout = str(sftp.lstat(os.path.join(path, fname))).split()[0]
        if 'd' not in lstatout:  # fname is file
            date = get_file_date(fname, filedate)
            if date >= begin and date <= end:
                filelist.append(path + fname)
        elif 'd' in lstatout:  # fname is dir
            filelist = filesInDir_sftp(path + fname + '/', sftp, filedate,
                                       begin, end, filelist)

    return filelist


def filesInDir_ftp(path, ftp, filedate, begin, end, filelist):
    """List all files in directory and subdirectories on an FTP server.

    Parameters
    ----------
    path : str
        Path to data on host.
    ftp : ftplib connection
        Connection to Server.
    filedate : dict
        Dict which points to the date fields in the filename.
    begin : datetime,
        Date from which on to download data.
    end : datetime
        Date until which to download data.
    filelist : list of str
        List of filepaths or empty list.

    Returns
    -------
    filelist : list
        List containing all files in directory and subdirectories
    """

    files = []
    ftp.cwd(path)
    try:
        ftp.retrlines('NLST', files.append)
    except socket.error:
        return False

    stats = []
    ftp.retrlines("LIST", stats.append)

    for idx, fname in enumerate(files):
        lstatout = stats[idx].split()[0]
        if 'd' not in lstatout:  # fname is file
            date = get_file_date(fname, filedate)
            if date >= begin and date <= end:
                filelist.append(path + fname)
        elif 'd' in lstatout:  # fname is dir
            filelist = filesInDir_ftp(path + fname + '/', ftp, filedate,
                                      begin, end, filelist)

    return filelist



'''
Created on Sep 15, 2014

@author: pydev
'''
import unittest
import os
import poets.image.geotiff as gt
import numpy as np
from PIL import Image


def curpath():
    pth, _ = os.path.split(os.path.abspath(__file__))
    return pth


class Test(unittest.TestCase):

    def setUp(self):
        self.testfile = os.path.join(curpath(), 'data', 'test.tiff')
        self.img = Image.open(self.testfile)
        self.region = 'IT'
        self.fileExtension = '.tiff'

        self.lon = 10.5
        self.lat = 40.0

        self.lon_min = 6.623967170715332
        self.lon_max = 18.514442443847656
        self.lat_min = 36.64916229248047
        self.lat_max = 47.094581604003906

        self.row = 31.922637612099024
        self.col = 21.84052307386542

    def tearDown(self):
        pass

    def test_lonlat2px_gt(self):
        row, col = gt.lonlat2px_gt(self.img, self.lon, self.lat,
                                         self.lon_min, self.lat_min,
                                         self.lon_max, self.lat_max)

        assert self.row == row
        assert self.col == col

    def test_px2lonlat_gt(self):
        lon_array = np.array([self.col])
        lat_array = np.array([self.row])
        lon, lat = gt.px2lonlat_gt(self.img, lon_array, lat_array,
                                         self.lon_min, self.lat_min,
                                         self.lon_max, self.lat_max)

        assert self.lat == lat[0]
        assert self.lon == lon[0]

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-06-04

import os
import shapefile
from shapely.geometry import MultiPolygon


class FipsError(Exception):
    pass


class Shape(object):
    """Provides geography information of a region/country given as shapefile.

    Parameters
    ----------
    code : str
        Identifier of the records in the shapefile; uses the first argument
        returned by shapefile.Reader as identifier, for the default shapefile,
        this would be the FIPS country code.
    shapefile : str, optional
        Path to shape file, uses "world country admin boundary shapefile" by
        default.

    Attributes
    ----------
    code : str
        Identifier of the selected record in the shapefile; uses the first
        argument returned by shapefile.Reader as identifier, for the default
        shapefile, this would be the FIPS country code.
    name : str
        Name of the selected record in the shapefile; uses the second argument
        returned by shapefile.Reader as identifier, for the default shapefile,
        this would be the FIPS country code.
    shpfile : str
        Path to the source shapefile.
    bbox : tuple
        Bounding box of the country.
    polygon : list of tuples
        Country boundary polygon.
    """

    def __init__(self, code, shapefile=None):
        self.code = code
        if shapefile is None:
            self.shpfile = os.path.join(os.path.dirname(__file__),
                                        'ancillary',
                                        'world_country_admin_boundary_'
                                        'shapefile_with_fips_codes')
        else:
            self.shpfile = shapefile
        rec, bbox, polygon = self._get_shape()
        self.name = rec[1]
        self.bbox = tuple(bbox)
        self.polygon = polygon

    def _get_shape(self):
        """Fetches shape and record information from shapefile.

        Returns
        -------
        record : list
            Record of the country.
        bbox : shapefile._Array
            Bounding box of the country.
        multipoly : shapely.geometry.MultiPolygon
            Country boundary polygon.

        Raises
        ------
        FipsError
            If FIPS code does not exist.
        """

        sf = shapefile.Reader(self.shpfile)
        pos = False
        for i, rec in enumerate(sf.records()):
            for attr in rec:
                if self.code == str(attr):
                    pos = i
                    break

        if pos is False:
            raise FipsError("Code '" + self.code + "' does not exist in "
                            "Shapefile")

        sh = sf.shapeRecord(pos)

        if len(sh.shape.parts) == 1:
            multipoly = [[sh.shape.points, []]]

        else:
            points = []
            for i in range(0, len(sh.shape.parts) - 1):
                points.append(sh.shape.points[sh.shape.parts[i]:
                                              sh.shape.parts[i + 1]])
            points.append(sh.shape.points[sh.shape.parts[-1]:])
            multipoly = []
            for i in range(0, len(points)):
                    multipoly.append([points[i], []])

        multipoly = MultiPolygon(multipoly)

        return sh.record, sh.shape.bbox, multipoly




# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-06-13

"""
This module provides functions for loading from and writing to NetCDF4 files.
"""

import os.path
import numpy as np
from netCDF4 import Dataset, date2num, num2date
from pygeogrids.netcdf import save_grid
from poets.grid import grids
from poets.timedate.dateindex import get_dtindex


def save_image(image, timestamp, region, metadata, dest_file, start_date,
               sp_res, nan_value=-99, shapefile=None, temp_res='dekad',
               compression=False):
    """Saves numpy.ndarray images as multidimensional netCDF4 file.

    Creates a datetimeindex over the whole period defined in the settings file

    Parameters
    ----------
    image : dict of numpy.ndarrays
        Input image.
    timestamp : datetime.datetime
        Timestamp of image.
    region : str, optional
        Identifier of the region in the shapefile. If the default shapefile is
        used, this would be the FIPS country code.
    metadata : dict
        NetCDF metadata from source file.
    dest_file : str
        Path to the output file.
    start_date : datetime.datetime
        First date of available data.
    sp_res :  int or float
        Spatial resolution of the grid.
    nan_value : int, optional
        Not a number value for dataset, defaults to -99.
    shapefile : str, optional
        Path to shape file, uses "world country admin boundary shapefile" by
        default.
    temp_res : string or int, optional
        Temporal resolution of the output NetCDF4 file, defaults to dekad.
    compression : bool, optional
        If True, ncfile compression is active.
    """

    if region == 'global':
        grid = grids.RegularGrid(sp_res)
    else:
        grid = grids.ShapeGrid(region, sp_res, shapefile)

    dest_file = dest_file

    if not os.path.isfile(dest_file):
        save_grid(dest_file, grid)

    dt = get_dtindex(temp_res, start_date)

    with Dataset(dest_file, 'r+', format='NETCDF4') as ncfile:

        if 'time' not in ncfile.dimensions.keys():
            ncfile.createDimension("time", None)

            if compression:
                times = ncfile.createVariable('time', 'uint16', ('time',),
                                              zlib=True, complevel=4)

            else:
                times = ncfile.createVariable('time', 'uint16', ('time',))

            times.units = 'days since ' + str(start_date)
            times.calendar = 'standard'
            times[:] = date2num(dt.tolist(), units=times.units,
                                calendar=times.calendar)

        else:
            times = ncfile.variables['time']

        dim = ('time', 'lat', 'lon')

        numdate = date2num(timestamp, units=times.units,
                           calendar=times.calendar)

        for key in image.keys():

            if key not in ncfile.variables.keys():

                if compression:
                    var = ncfile.createVariable(key, image[key].dtype.char,
                                                dim, zlib=True, complevel=4,
                                                fill_value=nan_value)
                else:
                    var = ncfile.createVariable(key, image[key].dtype.char,
                                                dim, fill_value=nan_value)
            else:
                var = ncfile.variables[key]

            if numdate in times[:]:
                var_index = np.where(times[:] == numdate)[0][0]
            else:
                times[times[:].size] = numdate
                var_index = times[:].size - 1

            var[var_index] = image[key]

            if metadata is not None:
                for item in metadata[key]:
                    if item in var.ncattrs():
                        continue
                    else:
                        var.setncattr(str(item), metadata[key][item])


def write_tmp_file(image, timestamp, region, metadata, dest_file, start_date,
                   sp_res, nan_value=-99, shapefile=None):
    """Saves numpy.ndarray images as multidimensional netCDF4 file.

    Parameters
    ----------
    image : dict of numpy.ndarrays
        Input image.
    timestamp : datetime.datetime
        Timestamp of image.
    region : str, optional
        Identifier of the region in the shapefile. If the default shapefile is
        used, this would be the FIPS country code.
    metadata : dict
        NetCDF metadata from source file.
    dest_file : str
        Path to the output file.
    start_date : datetime.datetime
        First date of available data.
    sp_res :  int or float
        Spatial resolution of the grid.
    nan_value : int, optional
        Not a number value for dataset, defaults to -99.
    shapefile : str, optional
        Path to shape file, uses "world country admin boundary shapefile" by
        default.
    """

    if region == 'global':
        grid = grids.RegularGrid(sp_res)
    else:
        grid = grids.ShapeGrid(region, sp_res, shapefile)

    if not os.path.isfile(dest_file):
        save_grid(dest_file, grid)

    with Dataset(dest_file, 'r+', format='NETCDF4') as ncfile:

        if 'time' not in ncfile.dimensions.keys():
            ncfile.createDimension("time", None)
            times = ncfile.createVariable('time', 'uint16', ('time',))
            times.units = 'days since ' + str(start_date)
            times.calendar = 'standard'

        else:
            times = ncfile.variables['time']

        numdate = date2num(timestamp, units=times.units,
                           calendar=times.calendar)

        dim = ('time', 'lat', 'lon')

        for key in image.keys():

            if key not in ncfile.variables.keys():
                var = ncfile.createVariable(key, image[key].dtype.char, dim,
                                            fill_value=nan_value)
            else:
                var = ncfile.variables[key]

            if times.shape[0] == 0:
                times[0] = numdate
                var_index = 0
            else:
                if numdate in times[:]:
                    var_index = np.where(times[:] == numdate)[0][0]
                else:
                    times[times[:].size] = numdate
                    var_index = times[:].size - 1

            var[var_index] = image[key]
            if metadata is not None:
                for item in metadata[key]:
                    if item in var.ncattrs():
                        continue
                    else:
                        var.setncattr(str(item), metadata[key][item])


def read_image(source_file, variables=None):
    """Reads data out of netCDF file and returns it as numpy.ndarray

    Parameters
    ----------
    source_file : str
        Path to source file.
    variables : list of str, optional
        Variables to read from file, reads all variables if not set

    Returns
    -------
    data : dict of numpy.arrays
        Source file.
    lon : numpy.array
        Longitudes of the source file.
    lat : numpy.array
        Latitudes of the source file.
    timestamp : datetime.date
        Timestamp of image.
    metadata : dict of strings
        Metadata from source netCDF file.
    """

    with Dataset(source_file, 'r', format='NETCDF4') as nc:

        if 'time' in nc.variables.keys():
            times = nc.variables['time']
            timestamp = num2date(times[:], units=times.units)[0]
        else:
            timestamp = None

        lon = np.copy(nc.variables['lon'])
        lat = np.copy(nc.variables['lat'])

        ncvars = nc.variables.keys()
        if 'gpi' in ncvars:
            ncvars.remove('gpi')
        if 'lat' in ncvars:
            ncvars.remove('lat')
        if 'lon' in ncvars:
            ncvars.remove('lon')
        if 'time' in ncvars:
            ncvars.remove('time')
        if 'subset_flag' in ncvars:
            ncvars.remove('subset_flag')

        data = {}
        metadata = {}

        for var in ncvars:
            if ((variables is not None and var in variables)
                or variables is None):
                dat = nc.variables[var][:][0]
                data[var] = dat[:]
                metadata[str(var)] = {}
                for attr in nc.variables[var].ncattrs():
                    if attr[0] != '_' and attr != 'scale_factor':
                        metadata[var][attr] = nc.variables[var].getncattr(attr)

    return data, lon, lat, timestamp, metadata


def clip_bbox(data, lon, lat, lon_min, lat_min, lon_max, lat_max):
    """Clips bounding box out of numpy.array and returns data as numpy.ndarray

    Parameters
    ----------
    data : dict of numpy.arrays
        Source file.
    lon : numpy.array
        Longitudes of source file
    lat : numpy.array
        Latitudes of source file
    lon_min : float
        Min longitude of bounding box.
    lat_min : float
        Min latitude of bounding box.
    lon_max : float
        Max longitude of bounding box.
    lat_max : float
        Max latitude of bounding box.

    Returns
    -------
    data_new : dict of numpy.arrays
        Clipped image.
    lon_new : numpy.array
        Longitudes of the clipped image.
    lat_new : numpy.array
        Latitudes of the clipped image.
    """

    lons = np.where((lon >= lon_min) & (lon <= lon_max))[0]
    lats = np.where((lat >= lat_min) & (lat <= lat_max))[0]

    lon_new = lon[lons.min():lons.max() + 1]
    lat_new = lat[lats.min():lats.max() + 1]

    data_new = {}

    for var in data.keys():
        data_new[var] = data[var][lats.min():lats.max() + 1,
                                  lons.min():lons.max() + 1]

    return data_new, lon_new, lat_new


def read_variable(source_file, variable, date, date_to=None):
    """Gets images from a netCDF file.

    Reads the image for a specific date. If date_to is given, it will return
    multiple images in a multidimensional numpy.ndarray

    Parameters
    ----------
    source_file : str
        Path to source file.
    variable : str
        Requested variable of image.
    date : datetime.datetime
        Date of the image, start date of data cube if date_to is set.
    date_to : datetime.date, optional
        End date of data cube to slice from NetCDF file.

    Returns
    -------
    image : numpy.ndarray
        Image for a specific date.
    lon : numpy.array
        Longitudes of the image.
    lat : numpy.array
        Latgitudes of the image.
    metadata : dict of strings
        Metadata from source netCDF file.
    """

    with Dataset(source_file, 'r', format='NETCDF4') as nc:
        times = nc.variables['time']
        lon = nc.variables['lon'][:]
        lat = nc.variables['lat'][:]
        var = nc.variables[variable]

        metadata = {}
        for attr in var.ncattrs():
            if attr[0] != '_' and attr != 'scale_factor':
                metadata[attr] = var.getncattr(attr)

        numdate = date2num(date, units=times.units, calendar=times.calendar)

        if date_to is None:
            image = var[np.where(times[:] == numdate)[0][0]]
        else:
            numdate_to = date2num(date_to, units=times.units,
                                  calendar=times.calendar)
            subset = np.where((times[:] >= numdate) & (times[:] <= numdate_to))
            image = var[subset]

    return image, lon, lat, metadata


def get_properties(src_file):
    """Gets variables, dimensions and time period from a netCDF file.

    Parameters
    ----------
    src_file : str
        Path to NetCDF file.

    Returns
    -------
    variables : list of str
        List of variables.
    dimensions : list of str
        Dimensions of the NetCDF file.
    period : list of datetime.datetime
        Date of first and last image in source file.
    """

    if not os.path.exists(src_file):
        return None, None, None

    with Dataset(src_file, 'r+', format='NETCDF4') as nc:
        variables = nc.variables.keys()
        dimensions = nc.dimensions.keys()
        time = nc.variables['time']
        period = [num2date(time[:].min(), units=time.units,
                           calendar=time.calendar),
                  num2date(time[:].max(), units=time.units,
                           calendar=time.calendar)]

        for dim in dimensions:
            variables.remove(dim)

    if 'gpi' in variables:
        variables.remove('gpi')
    if 'subset_flag' in variables:
        variables.remove('subset_flag')

    return variables, dimensions, period




# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-08-14

from datetime import datetime
from poets.poet import Poet
from poets.timedate.dateindex import get_dtindex
import numpy as np
import os
import shutil
import unittest


def curpath():
    pth, _ = os.path.split(os.path.abspath(__file__))
    return pth


class Test(unittest.TestCase):

    @classmethod
    def setUpClass(self):

        self.rootpath = os.path.join(curpath(), 'data', 'src_test')
        self.nan_value = -99
        self.regions = ['AU']
        self.spatial_resolution = 1
        self.temporal_resolution = 'week'
        self.start_date = datetime(2013, 1, 7)
        self.enddate = datetime(2013, 1, 13)
        self.testdate = datetime(2013, 1, 10)

        if os.path.exists(self.rootpath):
            shutil.rmtree(self.rootpath)

        os.mkdir(self.rootpath)
        os.mkdir(os.path.join(self.rootpath, 'DATA'))
        os.mkdir(os.path.join(self.rootpath, 'TMP'))

        self.poet = Poet(self.rootpath, self.regions, self.spatial_resolution,
                         self.temporal_resolution, self.start_date,
                         self.nan_value)

        # setup test png files
        self.pngdir = os.path.join(curpath(), 'data', 'testpngs')
        if os.path.exists(self.pngdir):
            shutil.rmtree(self.pngdir)
        os.mkdir(self.pngdir)
        dtindex = get_dtindex('day', self.start_date,
                              self.enddate)
        for dat in dtindex:
            year = str(dat.year)
            month = "%02d" % (dat.month)
            day = "%02d" % (dat.day)
            fname = ('test_' + year + '_' + month + '_' + day + '.png')
            shutil.copy(os.path.join(curpath(), 'data', 'test.png'),
                        os.path.join(self.pngdir, fname))

    @classmethod
    def tearDownClass(self):
        if os.path.exists(self.rootpath):
            shutil.rmtree(self.rootpath)
        if os.path.exists(self.pngdir):
            shutil.rmtree(self.pngdir)

    def test_init(self):
        assert len(self.poet.__dict__.keys()) == 14

    def test_add_source(self):
        name = 'TEST'
        filename = "test_{YYYY}_{MM}_{TT}.png"
        filedate = {'YYYY': (5, 9), 'MM': (10, 12), 'DD': (13, 15)}
        temp_res = 'daily'
        host = self.pngdir
        protocol = 'local'
        begin_date = datetime(2013, 1, 7)
        nan_value = 255
        data_range = (0, 254)
        valid_range = (0, 1)
        unit = "NDVI"
        colorbar = 'Greens'

        self.poet.add_source(name, filename, filedate, temp_res, host,
                             protocol, begin_date=begin_date,
                             nan_value=nan_value, valid_range=valid_range,
                             unit=unit, data_range=data_range,
                             colorbar=colorbar)

    def test_download(self):
        self.poet.download(begin=self.start_date, end=self.enddate)
        assert len(os.listdir(os.path.join(self.rootpath, 'RAWDATA'))) == 1

    def test_resample(self):
        self.poet.resample(begin=self.start_date, end=self.enddate)

    def test_fetch_data(self):
        self.poet.fetch_data(end=self.enddate)
        assert len(os.listdir(os.path.join(self.rootpath, 'DATA'))) == 1
        assert len(os.listdir(os.path.join(self.rootpath, 'RAWDATA'))) == 1

    def test_read_image(self):
        img, _, _, metadata = self.poet.read_image('TEST', self.testdate,
                                                   variable='TEST_dataset')
        assert img.shape == (3, 7)
        assert img[2, 4] == 0.69685039370078738
        assert img[0, 0] is np.ma.masked
        assert metadata is None

    def test_read_timeseries(self):
        location = (14.5, 46.5)

        ts = self.poet.read_timeseries('TEST', location,
                                       region=self.regions[0],
                                       variable='TEST_dataset')
        assert ts['TEST_dataset'][0] == 0.69685039370078738

    def test_get_gridpoints(self):
        gridpoints = self.poet.get_gridpoints()
        assert gridpoints[self.regions[0]].index.size == 12

    def test_get_variables(self):
        variables = self.poet.get_variables()
        assert variables == ['TEST_dataset']

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-06-13

import os
import numpy as np
import poets.image.netcdf as nc
import poets.image.hdf5 as h5
import matplotlib.path
from poets.image.imagefile import bbox_img
from poets.shape.shapes import Shape
from pytesmo.grid import resample
import scipy


imgfiletypes = ['.png', '.PNG', '.tif', '.tiff', '.TIF', '.TIFF', '.jpg',
                '.JPG', '.jpeg', '.JPEG', '.gif', '.GIF']


def resample_to_shape(source_file, region, sp_res, grid, prefix=None,
                      nan_value=None, dest_nan_value=None, variables=None,
                      shapefile=None):
    """
    Resamples images and clips country boundaries

    Parameters
    ----------
    source_file : str
        Path to source file.
    region : str
        Identifier of the region in the shapefile. If the default shapefile is
        used, this would be the FIPS country code.
    sp_res : int or float
        Spatial resolution of the shape-grid.
    grid : poets.grid.RegularGrid or poets.grid.ShapeGrid
        Grid to resample data to.
    prefix : str, optional
        Prefix for the variable in the NetCDF file, should be name of source
    nan_value : int, float, optional
        Not a number value of the original data as given by the data provider
    dest_nan_value : int or float, optional
        NaN value used in the final NetCDF file.
    variables : list of str, optional
        Variables to resample from original file.
    shapefile : str, optional
        Path to shape file, uses "world country admin boundary shapefile" by
        default.

    Returns
    -------
    res_data : dict of numpy.arrays
        resampled image
    dest_lon : numpy.array
        longitudes of the points in the resampled image
    dest_lat : numpy.array
        latitudes of the points in the resampled image
    gpis : numpy.array
        grid point indices
    timestamp : datetime.date
        date of the image
    metadata : dict
        Metadata derived from input file.
    """

    if prefix is not None:
        prefix += '_'

    fileExtension = os.path.splitext(source_file)[1].lower()

    if region == 'global':
        lon_min = -180
        lon_max = 180
        lat_min = -90
        lat_max = 90
    else:
        shp = Shape(region, shapefile)
        lon_min = shp.bbox[0]
        lon_max = shp.bbox[2]
        lat_min = shp.bbox[1]
        lat_max = shp.bbox[3]

    if fileExtension in ['.nc', '.nc3', '.nc4']:
        data_src, lon, lat, timestamp, metadata = nc.read_image(source_file,
                                                                variables)
        data, src_lon, src_lat = nc.clip_bbox(data_src, lon, lat, lon_min,
                                              lat_min, lon_max, lat_max)
    elif fileExtension in ['.h5']:
        data_src, lon, lat, timestamp, metadata = h5.read_image(source_file,
                                                                variables)
        data, src_lon, src_lat = nc.clip_bbox(data_src, lon, lat, lon_min,
                                              lat_min, lon_max, lat_max)
    elif fileExtension in imgfiletypes:
        data, src_lon, src_lat, timestamp, metadata = bbox_img(source_file,
                                                               region,
                                                               fileExtension,
                                                               shapefile)

    if nan_value is not None:
        for key in data.keys():
            data[key] = np.ma.array(data[key], mask=(data[key] == nan_value))

    src_lon, src_lat = np.meshgrid(src_lon, src_lat)

    lons = grid.arrlon[0:grid.shape[0]]
    dest_lon, dest_lat = np.meshgrid(lons, np.unique(grid.arrlat)[::-1])

    gpis = grid.get_bbox_grid_points(grid.arrlat.min(), grid.arrlat.max(),
                                     grid.arrlon.min(), grid.arrlon.max())

    search_rad = 180000 * sp_res

    data = resample.resample_to_grid(data, src_lon, src_lat, dest_lon,
                                     dest_lat, search_rad=search_rad)

    res_data = {}
    path = []

    if region != 'global':
        _, _, multipoly = shp._get_shape()
        for ring in multipoly:
            poly_verts = list(ring.exterior.coords)
            path.append(matplotlib.path.Path(poly_verts))

        coords = [grid.arrlon, grid.arrlat[::-1]]
        coords2 = np.zeros((len(coords[0]), 2))

        for idx in range(0, len(coords[0])):
            coords2[idx] = [coords[0][idx], coords[1][idx]]

        mask_old = path[0].contains_points(coords2)

    for key in data.keys():
        if variables is not None:
            if key not in variables:
                del metadata[key]
                continue

        if region != 'global':
            for ring in path:
                mask_new = (ring.contains_points(coords2))
                mask_rev = scipy.logical_or(mask_old, mask_new)
                mask_old = mask_rev

            mask_rev = mask_rev.reshape(dest_lon.shape)
            mask = np.invert(mask_rev)

            mask[data[key].mask == True] = True
        else:
            mask = data[key].mask

        if prefix is None:
            var = key
        else:
            var = prefix + key

        if metadata is not None:
            metadata[var] = metadata[key]
            if var != key:
                del metadata[key]
        res_data[var] = np.ma.masked_array(data[key], mask=np.copy(mask),
                                           fill_value=dest_nan_value)

        dat = np.copy(res_data[var].data)
        dat[mask == True] = dest_nan_value

        res_data[var] = np.ma.masked_array(dat, mask=np.copy(mask),
                                           fill_value=dest_nan_value)

    return res_data, dest_lon, dest_lat, gpis, timestamp, metadata


def average_layers(image, dest_nan_value):
    """
    Averages image layers, given as n-dimensional masked arrays to one image.

    Parameters
    ----------
    image : numpy.ma.MaskedArray
        Input image to average.
    dest_nan_value : int
        NaN value to be used.

    Returns
    -------
    avg_img : numpy.ma.MaskedArray
        Averaged image.
    """

    img = np.ma.masked_array(np.nanmean(image, axis=0),
                             fill_value=dest_nan_value)
    mask = img.mask
    data = np.copy(img.data)
    data[img.mask == True] = dest_nan_value
    avg_img = np.ma.masked_array(data, mask=mask,
                                 fill_value=dest_nan_value)

    return avg_img



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Maximilian Wonaschuetz
# Creation date: 2014-07-29

"""
This modules provides functions used while creating image overlays.
"""

from poets.shape.shapes import Shape
from poets.grid.grids import minmaxcoord


def bounds(country, shapefile=None):
    """
    Returns the bounding box, center coordinates and zoom level
    of a shape for web overlay purposes.

    Parameters
    ----------
    country : str
        FIPS country code (https://en.wikipedia.org/wiki/FIPS_country_code)
    shapefile : str, optional
        Paht to a custom shapefile.

    Returns
    -------
    lon_min : int
        Minimum longitude.
    lon_max : int
        Maximum longitude.
    lat_min : int
        Minimum latitude.
    lat_max : int
        Maximum latitude.
    c_lat : int
        Center latidute of image.
    c_lon : int
        Center longitude of image.
    zoom : int
        Zoom level for openlayers.
    """
    shp = Shape(country, shapefile)

    lon_min = shp.bbox[0]
    lon_max = shp.bbox[2]
    lat_min = shp.bbox[1]
    lat_max = shp.bbox[3]
    e_lon = lon_max - lon_min
    e_lat = lat_max - lat_min
    c_lon = lon_min + e_lon / 2
    c_lat = lat_min + e_lat / 2

    zoom = 0
    i = 1024  # To be replaced with the width of the map container!
    while i / 2 > e_lon:
        zoom += 1
        i = i / 2

    return lon_min, lon_max, lat_min, lat_max, c_lat, c_lon, zoom


def image_bounds(country, sp_res, shapefile=None):
    """
    Calculates bounding box, center coordinates and zoom level of an image
    for web overlay purposes.

    Parameters
    ----------
    country : str
        FIPS country code (https://en.wikipedia.org/wiki/FIPS_country_code)
    sp_res : numeric
        Spatial resolution of the image
    shapefile : str, optional
        Paht to a custom shapefile.

    Returns
    -------
    lon_min : int
        Minimum longitude.
    lon_max : int
        Maximum longitude.
    lat_min : int
        Minimum latitude.
    lat_max : int
        Maximum latitude.
    c_lat : int
        Center latidute of image.
    c_lon : int
        Center longitude of image.
    zoom : int
        Zoom level for openlayers.
    """

    lon_min, lon_max, lat_min, lat_max, c_lat, c_lon, zoom = bounds(country,
                                                                    shapefile)

    lon_min, lon_max = minmaxcoord(lon_min, lon_max, sp_res)
    lat_min, lat_max = minmaxcoord(lat_min, lat_max, sp_res)

    lon_min = lon_min-(sp_res/2)
    lon_max = lon_max+(sp_res/2)
    lat_min = lat_min-(sp_res/2)
    lat_max = lat_max+(sp_res/2)

    return lon_min, lon_max, lat_min, lat_max, c_lat, c_lon, zoom

if __name__ == "__main__":
    pass



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-08-14


import unittest
import os
import h5py
import numpy.testing as nptest
import numpy as np
import poets.grid.grids as gr
import matplotlib.pyplot as plt
from datetime import datetime
from poets.image.netcdf import save_image
from poets.image.resampling import resample_to_shape, average_layers


def curpath():
    pth, _ = os.path.split(os.path.abspath(__file__))
    return pth


class Test(unittest.TestCase):

    def setUp(self):
        self.sp_res = 60
        self.region = 'UG'
        self.timestamp = datetime.today()
        self.start_date = datetime.today()
        self.temp_res = 'day'
        self.fill_value = -99
        self.variable = 'data'

        # create image
        self.shape = (3, 6)
        self.mask = np.array([[1, 0, 1, 0, 1, 0], [0, 1, 0, 1, 0, 1],
                              [1, 0, 1, 0, 1, 0]])

        self.image = {}
        self.data = np.ma.array(np.ones(self.shape), mask=self.mask,
                                fill_value=self.fill_value)
        self.data.data[np.where(self.mask == 1)] = self.fill_value

        self.image['data'] = self.data
        self.image['data2'] = self.data * 2

        # create metadata
        self.metadata = {'data': {'Attribute1': 'Value1'},
                         'data2': {'Attribut2': 'Value2'},
                         'data3': {'Attribut3': 'Value3'}}

        if not os.path.exists(os.path.join(curpath(), 'data')):
            os.mkdir(os.path.join(curpath(), 'data'))

        self.grid = gr.ShapeGrid(self.region, self.sp_res)
        self.globalgrid = gr.RegularGrid(sp_res=self.sp_res)

        # Build NetCDF testfile
        self.ncfile = os.path.join(curpath(), 'data', 'test_nc.nc')
        if os.path.exists(self.ncfile):
            os.remove(self.ncfile)

        save_image(self.image, self.timestamp, 'global', self.metadata,
                   self.ncfile, self.start_date, self.sp_res,
                   temp_res=self.temp_res)

        # Build HDF5 testfile
        self.h5file = os.path.join(curpath(), 'data', 'tests_hdf5.h5')
        if os.path.exists(self.h5file):
            os.remove(self.h5file)

        with h5py.File(self.h5file, 'w') as hdf5_file:

            group = hdf5_file.create_group('group')
            for dataset_name in self.image.keys():
                attributes = self.metadata[dataset_name]
                write_data = self.image[dataset_name]
                dataset = group.create_dataset(dataset_name,
                                               write_data.shape,
                                               write_data.dtype,
                                               write_data)
                for key in attributes:
                    dataset.attrs[key] = attributes[key]

        # Build png Testfile
        self.pngfile = os.path.join(curpath(), 'data', 'test_png.png')
        if os.path.exists(self.pngfile):
            os.remove(self.pngfile)

        n = 60
        pngimg = np.kron(np.copy(self.data), np.ones((n, n)))
        pngimg[pngimg == self.fill_value] = np.NAN
        plt.imsave(self.pngfile, pngimg)

    def tearDown(self):

        if os.path.exists(self.ncfile):
            os.remove(self.ncfile)

        if os.path.exists(self.h5file):
            os.remove(self.h5file)

        if os.path.exists(self.pngfile):
            os.remove(self.pngfile)

    def test_resample_to_shape(self):

        # Test global with png
        data, dest_lon, dest_lat, gpis, _, _ = \
            resample_to_shape(self.pngfile, 'global', self.sp_res,
                              self.globalgrid, nan_value=self.fill_value,
                              dest_nan_value=self.fill_value, prefix='prefix')

        assert dest_lon.shape == self.data.shape
        assert dest_lat.shape == self.data.shape
        assert (gpis[0], gpis[-1]) == (0, 17)

        # Test local with hdf5
        resimg = {'data': np.ma.array([[self.data[1][3].data]],
                                      mask=self.mask[1][3],
                                      fill_value=self.fill_value),
                  'data2': np.ma.array([[self.data[1][3].data]],
                                       mask=self.mask[1][3],
                                       fill_value=self.fill_value)}

        data, _, _, _, _, metadata = \
            resample_to_shape(self.h5file, self.region, self.sp_res,
                              self.grid, nan_value=self.fill_value,
                              dest_nan_value=self.fill_value)

        nptest.assert_array_equal(data, resimg)
        assert metadata == {'data': self.metadata['data'],
                            'data2': self.metadata['data2']}

        # Test local with only one variable with netcdf
        reslon = np.array([[30.]])
        reslat = np.array([[0.]])
        resimg = {'data': np.ma.array([[self.data[1][3].data]],
                                      mask=self.mask[1][3],
                                      fill_value=self.fill_value)}

        data, dest_lon, dest_lat, gpis, timestamp, metadata = \
            resample_to_shape(self.ncfile, self.region, self.sp_res,
                              self.grid, nan_value=self.fill_value,
                              dest_nan_value=self.fill_value,
                              variables=['data'])

        timediff = self.timestamp - timestamp

        nptest.assert_array_equal(data, resimg)
        assert dest_lon[0][0] == reslon
        assert dest_lat[0][0] == reslat
        assert gpis[0] == 0
        assert timediff.days == 0
        assert metadata == {'data': self.metadata['data']}

    def test_average_layers(self):

        avgimg = self.image['data'] * 2

        image = np.ma.array([self.image['data'], self.image['data'] * 3])

        img = average_layers(image, self.fill_value)

        nptest.assert_array_equal(img, avgimg)

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Thomas Mistelbauer Thomas.Mistelbauer@geo.tuwien.ac.at
# Creation date: 2014-07-08

import unittest
import os
import numpy as np
import numpy.testing as nptest
from datetime import datetime
from poets.image.netcdf import save_image, write_tmp_file, clip_bbox, \
    read_image, get_properties, read_variable
from netCDF4 import Dataset


def curpath():
    pth, _ = os.path.split(os.path.abspath(__file__))
    return pth


class Test(unittest.TestCase):

    def setUp(self):
        self.sp_res = 90
        self.region = 'global'
        self.testfilename = os.path.join(curpath(), 'data', 'tests.nc')
        self.metadata = {'data': {'Attribute1': 'Value1'}}
        self.timestamp = datetime(2015, 1, 1)
        self.start_date = datetime(2015, 1, 1)
        self.temp_res = 'day'
        self.fill_value = -99
        self.variable = 'data'

        self.shape = (2, 4)
        self.mask = [[1, 0, 1, 0], [0, 1, 0, 1]]

        self.image = {}
        self.data = np.ma.array(np.ones(self.shape), mask=self.mask,
                                fill_value=self.fill_value)
        self.image['data'] = self.data

        self.bbox = np.ma.array(np.ones((1, 2)), mask=self.mask[0][0:2],
                                fill_value=self.fill_value)

        self.lon = np.array([-135., -45., 45., 135.])
        self.lat = np.array([45., -45.])
        self.lon_new = np.array([-135., -45.])
        self.lat_new = np.array([45.])

        if not os.path.exists(os.path.join(curpath(), 'data')):
            os.mkdir(os.path.join(curpath(), 'data'))

        if os.path.exists(self.testfilename):
            os.remove(self.testfilename)

    def tearDown(self):
        os.remove(self.testfilename)

    def test_save_image(self):

        save_image(self.image, self.timestamp, self.region, self.metadata,
                   self.testfilename, self.start_date, self.sp_res,
                   temp_res=self.temp_res)

        with Dataset(self.testfilename) as nc_data:
            data = nc_data.variables[self.variable]
            mask = np.array(self.mask, dtype=bool)
            nptest.assert_array_equal(self.data, data[0])
            nptest.assert_array_equal(mask, data[0].mask)
            assert data.getncattr('_FillValue') == self.fill_value
            assert data.getncattr('Attribute1') == \
                self.metadata[self.variable]['Attribute1']

    def test_write_tmp_file(self):
        write_tmp_file(self.image, self.timestamp, self.region, self.metadata,
                       self.testfilename, self.start_date, self.sp_res)

        with Dataset(self.testfilename) as nc_data:
            data = nc_data.variables[self.variable]
            mask = np.array(self.mask, dtype=bool)
            nptest.assert_array_equal(self.data, data[0])
            nptest.assert_array_equal(mask, data[0].mask)
            assert data.getncattr('_FillValue') == self.fill_value
            assert data.getncattr('Attribute1') == \
                self.metadata[self.variable]['Attribute1']

    def test_read_image(self):
        write_tmp_file(self.image, self.timestamp, self.region, self.metadata,
                       self.testfilename, self.start_date, self.sp_res)

        _, _, _, timestamp, metadata = read_image(self.testfilename)

        timediff = self.timestamp - timestamp

        assert timediff.days == 0
        assert metadata == self.metadata

    def test_clip_bbox(self):

        lon_min = -180
        lon_max = 0
        lat_min = 0
        lat_max = 90

        write_tmp_file(self.image, self.timestamp, self.region, self.metadata,
                       self.testfilename, self.start_date, self.sp_res)

        data_src, lon, lat, _, _ = read_image(self.testfilename)
        data, lon_new, lat_new = clip_bbox(
            data_src, lon, lat, lon_min, lat_min, lon_max, lat_max)

        nptest.assert_array_equal(self.bbox, data[self.variable])
        nptest.assert_array_equal(self.lon_new, lon_new)
        nptest.assert_array_equal(self.lat_new, lat_new)

    def test_read_variable(self):

        save_image(self.image, self.timestamp, self.region, self.metadata,
                   self.testfilename, self.start_date, self.sp_res,
                   temp_res=self.temp_res)

        image, lon, lat, metadata = read_variable(self.testfilename,
                                                  self.variable,
                                                  self.timestamp)

        nptest.assert_array_equal(image, self.data)
        nptest.assert_array_equal(lon, self.lon)
        nptest.assert_array_equal(lat, self.lat)
        assert metadata == self.metadata[self.variable]

    def test_get_properties(self):

        save_image(self.image, self.timestamp, self.region, self.metadata,
                   self.testfilename, self.start_date, self.sp_res,
                   temp_res=self.temp_res)

        variables, dimensions, period = get_properties(self.testfilename)

        timediff1 = self.timestamp - period[0]

        assert self.variable in variables
        assert dimensions == ['lat', 'lon', 'time']
        assert timediff1.days == 0


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# Copyright (c) 2014, Vienna University of Technology (TU Wien), Department
# of Geodesy and Geoinformation (GEO).
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the Vienna University of Technology - Department of
#   Geodesy and Geoinformation nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Author: Isabella Pfeil, isy.pfeil@gmx.at
# Creation date: 2014-08-14

import unittest
import os
import poets.io.fileformats as ff


class Test(unittest.TestCase):

    def setUp(self):
        self.formats = ['.nc', '.nc4', '.h5', '.tiff', '.tif', '.png', '.jpg',
                        '.jpeg', '.gif']
        self.filelist = ['test.doc', 'test.nc', 'test.abc']

    def tearDown(self):
        pass

    def test_check_supported(self):
        for fExt in self.formats:
            test = ff.check_supported(os.path.join('test.' + fExt))
            self.failUnless(test == True)

    def test_select_file(self):
        fname = 'test.nc'
        flist = ff.select_file(self.filelist)

        assert flist == fname

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
